@@ -149,7 +149,7 @@ get_component_from_SDA <- function(WHERE = NULL, duplicates = FALSE, childs = TR
     idx    <- d.component$cokey %in% cokeys
 
     assign('component.ecosite.problems', value = cokeys, envir = soilDB.env)
-    message("-> QC: multiple ecosites linked to 1 component use `get('component.ecosite.problems', envir = soilDB.env)` for related cokey values")
+    message("-> QC: multiple ecosites linked to 1 component\n\tUse `get('component.ecosite.problems', envir = soilDB.env)` for component keys (cokey)")
 
     nodups <- {
       d.component[idx, ] ->.;
@@ -222,25 +222,25 @@ get_component_from_SDA <- function(WHERE = NULL, duplicates = FALSE, childs = TR
   SELECT
   cokey, cosurffragskey, sfragcov_r, para, nonpara,
   -- fragments
-  CASE WHEN   fragsize_r2 >= 2  AND fragsize_r2 <= 5   AND shape = 'nonflat'
+  CASE WHEN   fragsize_r2 >= 2  AND fragsize_r2 < 5   AND shape = 'nonflat'
        THEN 1 ELSE NULL
        END fine_gravel,
-  CASE WHEN   fragsize_r2 >= 2  AND fragsize_r2 <= 76  AND shape = 'nonflat'
+  CASE WHEN   fragsize_r2 >= 2  AND fragsize_r2 < 75  AND shape = 'nonflat'
        THEN 1 ELSE NULL
        END gravel,
-  CASE WHEN   fragsize_r2 > 76  AND fragsize_r2 <= 250 AND shape = 'nonflat'
+  CASE WHEN   fragsize_r2 >= 75 AND fragsize_r2 < 250 AND shape = 'nonflat'
        THEN 1 ELSE NULL
        END cobbles,
-  CASE WHEN ((fragsize_r2 > 250 AND fragsize_r2 <= 600 AND shape = 'nonflat') OR
-            (fragsize_r2 >= 380 AND fragsize_r2 < 600 AND shape = 'flat'))
+  CASE WHEN ((fragsize_r2 >= 250 AND fragsize_r2 < 600 AND shape = 'nonflat') OR
+             (fragsize_r2 >= 380 AND fragsize_r2 < 600 AND shape = 'flat'))
        THEN 1 ELSE NULL END stones,
-  CASE WHEN   fragsize_r2 > 600
+  CASE WHEN   fragsize_r2 >= 600
        THEN 1 ELSE NULL
        END boulders,
-  CASE WHEN   fragsize_r2 >= 2  AND fragsize_r2 <= 150 AND shape = 'flat'
+  CASE WHEN   fragsize_r2 >= 2   AND fragsize_r2 < 150 AND shape = 'flat'
        THEN 1 ELSE NULL
        END channers,
-  CASE WHEN   fragsize_r2 > 150 AND fragsize_r2 <= 380 AND shape = 'flat'
+  CASE WHEN   fragsize_r2 >= 150 AND fragsize_r2 < 380 AND shape = 'flat'
        THEN 1 ELSE NULL
        END flagstones,
   CASE WHEN   fragsize_r2 IS NULL
@@ -619,15 +619,15 @@ get_chorizon_from_SDA <- function(WHERE = NULL, duplicates = FALSE,
                           SELECT
                           cokey, chkey, chfragskey, fragvol_r, para, nonpara,
                           -- fragments
-                          CASE WHEN   fragsize_r2 >= 2  AND fragsize_r2 <= 5   AND shape = 'nonflat' THEN 1 ELSE NULL END fine_gravel,
-                          CASE WHEN   fragsize_r2 >= 2  AND fragsize_r2 <= 76  AND shape = 'nonflat' THEN 1 ELSE NULL END gravel,
-                          CASE WHEN   fragsize_r2 > 76  AND fragsize_r2 <= 250 AND shape = 'nonflat' THEN 1 ELSE NULL END cobbles,
-                          CASE WHEN ((fragsize_r2 > 250 AND fragsize_r2 <= 600 AND shape = 'nonflat') OR
-                          (fragsize_r2 >= 380 AND fragsize_r2 < 600 AND shape = 'flat'))
+                          CASE WHEN   fragsize_r2 >= 2   AND fragsize_r2 < 5   AND shape = 'nonflat' THEN 1 ELSE NULL END fine_gravel,
+                          CASE WHEN   fragsize_r2 >= 2   AND fragsize_r2 < 75  AND shape = 'nonflat' THEN 1 ELSE NULL END gravel,
+                          CASE WHEN   fragsize_r2 >= 75  AND fragsize_r2 < 250 AND shape = 'nonflat' THEN 1 ELSE NULL END cobbles,
+                          CASE WHEN ((fragsize_r2 >= 250 AND fragsize_r2 < 600 AND shape = 'nonflat') OR
+                          (fragsize_r2 >= 380 AND fragsize_r2 <= 600 AND shape = 'flat'))
                           THEN 1 ELSE NULL END stones,
-                          CASE WHEN   fragsize_r2 > 600 THEN 1 ELSE NULL END boulders,
-                          CASE WHEN   fragsize_r2 >= 2  AND fragsize_r2 <= 150 AND shape = 'flat' THEN 1 ELSE NULL END channers,
-                          CASE WHEN   fragsize_r2 > 150 AND fragsize_r2 <= 380 AND shape = 'flat' THEN 1 ELSE NULL END flagstones,
+                          CASE WHEN   fragsize_r2 >= 600 THEN 1 ELSE NULL END boulders,
+                          CASE WHEN   fragsize_r2 >= 2   AND fragsize_r2 < 150 AND shape = 'flat' THEN 1 ELSE NULL END channers,
+                          CASE WHEN   fragsize_r2 >= 150 AND fragsize_r2 < 380 AND shape = 'flat' THEN 1 ELSE NULL END flagstones,
                           CASE WHEN   fragsize_r2 IS NULL                                         THEN 1 ELSE NULL END unspecified
 
                           FROM
@@ -707,12 +707,12 @@ get_chorizon_from_SDA <- function(WHERE = NULL, duplicates = FALSE,
 
 
 
-#' Get SSURGO/STATSGO2 Mapunit Data from Soil Data Access
+#' @title Get SSURGO/STATSGO2 Mapunit Data from Soil Data Access
 #'
-#' Functions to download and flatten commonly used tables and from Soil Data
+#' @description Functions to download and flatten commonly used tables and from Soil Data
 #' Access, and create soil profile collection objects (SPC).
 #'
-#' These functions return data from Soil Data Access with the use of a simple
+#' @details These functions return data from Soil Data Access with the use of a simple
 #' text string that formatted as an SQL WHERE clause (e.g. \code{WHERE =
 #' "areasymbol = 'IN001'"}. All functions are SQL queries that wrap around
 #' \code{SDAquery()} and format the data for analysis.
@@ -732,7 +732,9 @@ get_chorizon_from_SDA <- function(WHERE = NULL, duplicates = FALSE,
 #' data-gaps and NULL rock fragment values should be interpreted as NULLs. Set
 #' \code{nullFragsAreZero = TRUE} in those cases where NULL rock fragment
 #' values should be interpreted as 0.
-#'
+#' 
+#' Additional examples can be found in the [Soil Data Access (SDA) Tutorial](http://ncss-tech.github.io/AQP/soilDB/SDA-tutorial.html)
+#' 
 #' @aliases fetchSDA get_legend_from_SDA get_lmuaoverlap_from_SDA
 #' get_mapunit_from_SDA get_component_from_SDA get_chorizon_from_SDA
 #' get_cosoilmoist_from_SDA get_cointerp_from_SDA
@@ -757,120 +759,6 @@ get_chorizon_from_SDA <- function(WHERE = NULL, duplicates = FALSE,
 #' @author Stephen Roecker
 #' @seealso \link{SDA_query}
 #' @keywords manip
-#' @examples
-#'
-#' \donttest{
-#'
-#'
-#' if (requireNamespace("curl") &
-#'   curl::has_internet() &
-#'   require(aqp) &
-#'   require("ggplot2") &
-#'   require("gridExtra") &
-#'   require("viridisLite")
-#' ) {
-#'
-#'   # query soil components by areasymbol and musym
-#'   test = fetchSDA(WHERE = "areasymbol = 'IN005' AND musym = 'MnpB2'")
-#'
-#'
-#'   # profile plot
-#'   plot(test)
-#'
-#'
-#'   # convert the data for depth plot
-#'   clay_slice = horizons(slice(test, 0:200 ~ claytotal_l + claytotal_r + claytotal_h))
-#'   names(clay_slice) <- gsub("claytotal_", "", names(clay_slice))
-#'
-#'   om_slice = horizons(slice(test, 0:200 ~ om_l + om_r + om_h))
-#'   names(om_slice) = gsub("om_", "", names(om_slice))
-#'
-#'   test2 = rbind(data.frame(clay_slice, var = "clay"),
-#'                 data.frame(om_slice, var = "om")
-#'   )
-#'
-#'   h = merge(test2, site(test)[c("nationalmusym", "cokey", "compname", "comppct_r")],
-#'             by = "cokey",
-#'             all.x = TRUE
-#'   )
-#'
-#'   # depth plot of clay content by soil component
-#'   gg_comp <- function(x) {
-#'     ggplot(x) +
-#'       geom_line(aes(y = r, x = hzdept_r)) +
-#'       geom_line(aes(y = r, x = hzdept_r)) +
-#'       geom_ribbon(aes(ymin = l, ymax = h, x = hzdept_r), alpha = 0.2) +
-#'       xlim(200, 0) +
-#'       xlab("depth (cm)") +
-#'       facet_grid(var ~ nationalmusym + paste(compname, comppct_r)) +
-#'       coord_flip()
-#'   }
-#'   g1 <- gg_comp(subset(h, var == "clay"))
-#'   g2 <- gg_comp(subset(h, var == "om"))
-#'
-#'   grid.arrange(g1, g2)
-#'
-#'
-#'   # query cosoilmoist (e.g. water table data) by mukey
-#'   x <- get_cosoilmoist_from_SDA(WHERE = "mukey = '1395352'")
-#'
-#'   ggplot(x, aes(x = as.integer(month), y = dept_r, lty = status)) +
-#'     geom_rect(aes(xmin = as.integer(month), xmax = as.integer(month) + 1,
-#'                   ymin = 0, ymax = max(x$depb_r),
-#'                   fill = flodfreqcl)) +
-#'     geom_line(cex = 1) +
-#'     geom_point() +
-#'     geom_ribbon(aes(ymin = dept_l, ymax = dept_h), alpha = 0.2) +
-#'     ylim(max(x$depb_r), 0) +
-#'     xlab("month") + ylab("depth (cm)") +
-#'     scale_x_continuous(breaks = 1:12, labels = month.abb, name="Month") +
-#'     facet_wrap(~ paste0(compname, ' (', comppct_r , ')')) +
-#'     ggtitle(paste0(x$nationalmusym[1],
-#'                    ': Water Table Levels from Component Soil Moisture Month Data'))
-#'
-#'
-#'
-#'   # query all Miami major components
-#'   s <- get_component_from_SDA(WHERE = "compname = 'Miami' \n
-#'                 AND majcompflag = 'Yes' AND areasymbol != 'US'")
-#'
-#'
-#'   # landform vs 3-D morphometry
-#'   test <- {
-#'     subset(s, ! is.na(landform) | ! is.na(geompos)) ->.;
-#'     split(., .$drainagecl, drop = TRUE) ->.;
-#'     lapply(., function(x) {
-#'       test = data.frame()
-#'       test = as.data.frame(table(x$landform, x$geompos))
-#'       test$compname   = x$compname[1]
-#'       test$drainagecl = x$drainagecl[1]
-#'       names(test)[1:2] <- c("landform", "geompos")
-#'       return(test)
-#'     }) ->.;
-#'     do.call("rbind", .) ->.;
-#'     .[.$Freq > 0, ] ->.;
-#'     within(., {
-#'       landform = reorder(factor(landform), Freq, max)
-#'       geompos  = reorder(factor(geompos),  Freq, max)
-#'       geompos  = factor(geompos, levels = rev(levels(geompos)))
-#'     }) ->.;
-#'   }
-#'   test$Freq2 <- cut(test$Freq,
-#'                     breaks = c(0, 5, 10, 25, 50, 100, 150),
-#'                     labels = c("<5", "5-10", "10-25", "25-50", "50-100", "100-150")
-#'   )
-#'   ggplot(test, aes(x = geompos, y = landform, fill = Freq2)) +
-#'     geom_tile(alpha = 0.5) + facet_wrap(~ paste0(compname, "\n", drainagecl)) +
-#'     discrete_scale("colour", "viridis", function(n) viridisLite::viridis(n)) +
-#'     theme(aspect.ratio = 1, axis.text.x = element_text(angle = 45, hjust = 1, vjust = 1)) +
-#'     ggtitle("Landform vs 3-D Morphometry for Miami Major Components on SDA")
-#'
-#'
-#' }
-#'
-#'
-#'
-#' }
 #'
 #' @export fetchSDA
 fetchSDA <- function(WHERE = NULL, duplicates = FALSE, childs = TRUE,
@@ -946,7 +834,7 @@ fetchSDA <- function(WHERE = NULL, duplicates = FALSE, childs = TRUE,
 
   # print any messages on possible data quality problems:
   if (exists('component.hz.problems', envir=soilDB.env))
-    message("-> QC: horizon errors detected, use `get('component.hz.problems', envir=soilDB.env)` for related cokey values")
+    message("-> QC: horizon errors detected, use `get('component.hz.problems', envir=soilDB.env)` for component keys (cokey)")
 
   # done, return SPC
   return(f.chorizon)
