@@ -1,7 +1,5 @@
-get_phfmp_from_NASIS_db <- function(SS = TRUE, stringsAsFactors = default.stringsAsFactors()) {
-  # must have RODBC installed
-  if(!requireNamespace('RODBC'))
-    stop('please install the `RODBC` package', call.=FALSE)
+get_phfmp_from_NASIS_db <- function(SS = TRUE, stringsAsFactors = default.stringsAsFactors(), dsn = NULL) {
+
 
   # because of alias with fetchNASIS cannot allow setting attr
   # also, attr is a free-form field, so not terribly useful -- consider SQL LIKE?
@@ -13,20 +11,19 @@ get_phfmp_from_NASIS_db <- function(SS = TRUE, stringsAsFactors = default.string
     q <- "SELECT * FROM phfmp_View_1;"
   #}
 
-    channel <- .openNASISchannel()
-    if (channel == -1)
-      return(data.frame())
+  channel <- dbConnectNASIS(dsn)
+
+  if (inherits(channel, 'try-error'))
+    return(data.frame())
 
   # toggle selected set vs. local DB
-  if(SS == FALSE) {
+  if (SS == FALSE) {
     q <- gsub(pattern = '_View_1', replacement = '', x = q, fixed = TRUE)
   }
 
-  # exec queries
-  d <- RODBC::sqlQuery(channel, q, stringsAsFactors=FALSE)
-
-  RODBC::odbcClose(channel)
+  # exec query
+  d <- dbQueryNASIS(channel, q)
 
   # field measured properties, long format
-  return(uncode(d, stringsAsFactors = stringsAsFactors))
+  return(uncode(d, stringsAsFactors = stringsAsFactors, dsn = dsn))
 }
